/* Copyright (c) 2018 vesoft inc. All rights reserved.
 *
 * This source code is licensed under Apache 2.0 License.
 */

#include "codec/RowReader.h"

#include "codec/RowReaderWrapper.h"
#include "common/base/Base.h"

namespace nebula {

/*********************************************
 *
 * class RowReader::Cell
 *
 ********************************************/
Value RowReader::Cell::value() const noexcept {
  return iter_->reader_->getValueByIndex(iter_->index_);
}

/*********************************************
 *
 * class RowReader::Iterator
 *
 ********************************************/

bool RowReader::Iterator::operator==(const Iterator& rhs) const noexcept {
  return reader_ == rhs.reader_ && index_ == rhs.index_;
}

const RowReader::Cell& RowReader::Iterator::operator*() const noexcept {
  return cell_;
}

const RowReader::Cell* RowReader::Iterator::operator->() const noexcept {
  return &cell_;
}

RowReader::Iterator& RowReader::Iterator::operator++() {
  if (index_ < reader_->numFields()) {
    ++index_;
  }
  return *this;
}

/*********************************************
 *
 * class RowReader
 *
 ********************************************/

bool RowReader::resetImpl(meta::SchemaProviderIf const* schema, folly::StringPiece row) noexcept {
  schema_ = schema;
  data_ = row;

  endIter_.reset(schema_->getNumFields());
  return true;
}

}  // namespace nebula
